#!
# Build a dictionary from a MLF
# steps:
# - scan a MLF and look for entries with one word
# - count the occurences of each word
# - discard  words with less then MinCount occurences
# - build a wordlist from all others 
#
# input: 
#  MLF
# output:
#  dictonary
#  list of users
# 
#  Version
#    1.0       Mai 2005    Stephan Euler
#

$minCount = 6;       # minimum number of utterances of a word
$vocDir  = "arbeit\\htkfiles\\dicts";
$vocFile = $vocDir . "\\vokabular";
$userDir = "arbeit\\lists";
$userFile = $userDir . "\\users";

system( "mkdir $vocDir" );
system( "mkdir $userDir" );

$mlfFile = shift @ARGV;
$wordStatFile = "word_statistic.dat";

print "Working on $mlfFile\n";
open(IN, '<' .  $mlfFile)  || die "could not open $mlfFile";  

FILE:
while( <IN> ) {
  if( /^\#\!MLF\!\#/ ) { 
      next FILE;
  }
  @parts = split( /[\\\/]/, $_ );
  ++$users{ $parts[$parts-2] };    # add count for this user
  $trans ="";
  $wordCount = 0;
  while( <IN> ) {
    if( /^\./ ) {                   # End of entry ?
      # got an entry, now check it
      $trans =~ s/[\n\r\f]//g;
      $trans =~ s/ $//g;
      if( $wordCount == 1 ) {
	++$dict{$trans};
      } else {
	print "Ignore $trans\n"
      }
      next FILE;
    } else {
      ++$allDict{$_};
      $trans .= $_ . " ";
      ++$wordCount;
    }
  }
}

open(OUT, '>' .  $vocFile)  || die "could not open $vocFile";  
open(OUT2, '>' .  $userFile)  || die "could not open $userFile";  

print "*********************************************\n";
print "**************** Words **********************\n";
foreach $key ( sort keys(%dict) ) {
  print "$dict{$key} ";
  if( $dict{$key} < $minCount ) {
    print "\t discard  ";
  }
  else {
    print OUT "$key\n";
    print "\t use      ";
  }
  print "$key \n";
}

open(OUTWS, '>' .  $wordStatFile)  || die "could not open $wordStatFile";  
foreach $key ( sort keys(%allDict) ) {
  print OUTWS " $allDict{$key} \t  $key";
}
close OUTWS;

print "*********************************************\n";
print "**************** Users **********************\n";
foreach $key ( sort keys(%users) ) {
  print "$users{$key} \t $key\n";
  print OUT2  "$key\n";
}
close IN;
close OUT;
close OUT2;

$cmd = "HLStats -c 10000 $vocFile $mlfFile ";
print "$cmd\n";
system( $cmd );

